;/*******************************************************************************
;* DISCLAIMER
;* This software is supplied by Renesas Electronics Corporation and is only
;* intended for use with Renesas products. No other uses are authorized. This
;* software is owned by Renesas Electronics Corporation and is protected under
;* all applicable laws, including copyright laws.
;* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING
;* THIS SOFTWARE, WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT
;* LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
;* AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
;* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS
;* ELECTRONICS CORPORATION NOR ANY OF ITS AFFILIATED COMPANIES SHALL BE LIABLE
;* FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR
;* ANY REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE
;* BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
;* Renesas reserves the right, without notice, to make changes to this software
;* and to discontinue the availability of this software. By using this software,
;* you agree to the additional terms and conditions found by accessing the
;* following link:
;* http://www.renesas.com/disclaimer
;* Copyright (C) 2012 - 2014 Renesas Electronics Corporation. All rights reserved.
;*******************************************************************************/
;/*******************************************************************************
;* File Name   : reset_handler.s
;* $Rev: 817 $
;* $Date:: 2014-04-18 16:19:12 +0900#$
;* Description : Reset handler
;*******************************************************************************/


; Standard definitions of mode bits and interrupt (I & F) flags in PSRs
USR_MODE        EQU     0x10
FIQ_MODE        EQU     0x11
IRQ_MODE        EQU     0x12
SVC_MODE        EQU     0x13
ABT_MODE        EQU     0x17
UND_MODE        EQU     0x1b
SYS_MODE        EQU     0x1f
Thum_bit        EQU     0x20            ; CPSR/SPSR Thumb bit


;==================================================================
; Entry point for the Reset handler
;==================================================================
    PRESERVE8
    AREA RESET_HANDLER, CODE, READONLY

    IMPORT    ||Image$$SVC_STACK$$ZI$$Limit||       ; Linker symbol from scatter file

    IMPORT    init_spibsc_init2_section
    IMPORT    spibsc_init2

    IMPORT    spibsc_init1
    IMPORT    init_spibsc_init1_section
    IMPORT    VbarInit

    EXPORT  reset_handler
    EXPORT  undefined_handler
    EXPORT  svc_handler
    EXPORT  prefetch_handler
    EXPORT  abort_handler
    EXPORT  reserved_handler
    EXPORT    irq_handler
    EXPORT    fiq_handler

;******************************************************************************
; Function Name : reset_handler
; Description   : This function is the assembler function executed after reset
;               : cancellation. After initial setting for the stack pointer or 
;               : the MMU and reset cancellation, executes initial setting for 
;               : the minimum required peripheral functions. Calls the __main
;               : of the standard library function to execute the main function.
;******************************************************************************
reset_handler   FUNCTION {}


;==================================================================
; Clock Setting
;==================================================================
    ; Set standby_mode_en of Power Control Register
    LDR  r0, =0x3FFFFF80
    LDR  r1, =0x00000001
    STR  r1, [r0]
    LDR  r0, [r0]                    ;;; dummy read

    ; FRQCR = 0x1035  PLL(x30), I:G:B:P1:P0 = 30:20:10:5:5/2
    LDR  r0, =0xFCFE0010
    LDR  r1, =0x1035
    STRH r1, [r0]
    LDR  r0, [r0]                    ;;; dummy read

    ; FRQCR2 = 0x0001
    LDR  r0, =0xFCFE0014
    LDR  r1, =0x0001
    STRH r1, [r0]
    LDR  r0, [r0]                    ;;; dummy read


;==================================================================
; Disable cache and MMU in case it was left enabled from an earlier run
; This does not need to be done from a cold reset 
;==================================================================
    MRC  p15, 0, r0, c1, c0, 0       ;;; Read CP15 System Control register (SCTLR)
    BIC  r0, r0, #(0x1 << 13)        ;;; Set Low Vectors
    BIC  r0, r0, #(0x1 << 12)        ;;; Clear I bit 12 to disable I Cache
    BIC  r0, r0, #(0x1 <<  2)        ;;; Clear C bit  2 to disable D Cache
    BIC  r0, r0, #0x1                ;;; Clear M bit  0 to disable MMU
    MCR  p15, 0, r0, c1, c0, 0       ;;; Write value back to CP15 System Control register

    ; VBAR(Vector Base Address Register) = H'1800_0000
    BL   VbarInit

;==================================================================
; Setting up Stack Area
;==================================================================
                                    ;;; SVC Mode(Default)
    LDR  sp, =||Image$$SVC_STACK$$ZI$$Limit||

;==================================================================
; s-flash boot setting
;==================================================================
    BL   init_spibsc_init1_section
    BL   spibsc_init1

    BL   init_spibsc_init2_section
    BL   spibsc_init2


reset_handler_end
    B    reset_handler_end           ;;; Looping in this address

    ENDFUNC

Literals2
    LTORG


;==================================================================
; Other Handler
;==================================================================
undefined_handler
    B   undefined_handler           ;;; Looping in this address

svc_handler
    B   svc_handler                 ;;; Looping in this address

prefetch_handler
    B   prefetch_handler            ;;; Looping in this address

abort_handler
    B   abort_handler               ;;; Looping in this address

reserved_handler
    B   reserved_handler            ;;; Looping in this address

irq_handler
    B    irq_handler                 ;;; Looping in this address

fiq_handler
    B    fiq_handler                 ;;; Looping in this address

    END
